#include <stdio.h>
#include <netdb.h>
#include <stdlib.h>
#include "fopensock.h"
#include "gethttp.h"

/* Om bijvoorbeeld http://zeus.rug.ac.be/C/ te lezen moet
 * host wijzen naar een hostent die met zeus.rug.ac.be overeenkomt
 * en url de string "/C/" bevatten.
 *
 * voorbeeld: gethttp(gethostbyname("zeus.rug.ac.be"),"/~geert/",NULL);
 *
 * Deze functie voegt aan de body een extra nulbyte toe; die nulbyte
 * wordt niet meegeteld in de bodyLength.
 */

struct httpDocument *gethttp(struct hostent *host,char *url,void (*callback)(long)) {
        FILE *sock;
        int letter;
	char *proxy = getenv("PROXY");
	struct httpDocument *result = malloc(sizeof(struct httpDocument));
	long bufferGrootte;	/* er is plaats voor zoveel bytes */
	long bufferGevuld;	/* zoveel bytes zijn er al opgebruikt */
	char *hostname;	/* alleen gebruikt als PROXY gezet is */

	if (!result) return NULL;

	if (proxy) {
		/* verbinden met PROXY op poort PROXYPORT */
		int proxyport = 8080;
		char *port = getenv("PROXYPORT");
		if (port) proxyport = atoi(port);
		/* gethostbyname() kan 'host' vernietigen
		 * dus kopietje nemen van wat we nog nodig hebben */
		hostname = malloc(strlen(host->h_name) + 1);
		strcpy(hostname,host->h_name);
		sock = fopenSocket(gethostbyname(proxy),proxyport);
	} else	/* rechtstreeks verbinden */
		sock = fopenSocket(host,80);
	if (!sock) {
		free(result);
		return NULL;
	}
	if (proxy) {
	        fprintf(sock,"GET http://%s%s HTTP/1.0\n",hostname,url);
	} else
	        fprintf(sock,"GET %s HTTP/1.0\n",url);
	fprintf(sock,"User-Agent: Mozilla/3.01\n");
	fprintf(sock,"\n");	/* einde header */
	/* header lezen */
	result->header = malloc(bufferGrootte = 1000);
	if (!result->header) {
		free(result);
		return NULL;
	}
	bufferGevuld = 0;
        while ((letter = fgetc(sock)) != EOF) {
		/* Einde regel wordt als \r\n doorgestuurd.
		 * Gooi de \r weg. */
		if (letter == '\r') continue;
		result->header[bufferGevuld++] = letter;
		if (bufferGevuld == bufferGrootte) {	/* buffer is vol */
			result->header = realloc(result->header,bufferGrootte+=1000);
			if (!result->header) {
				free(result);
				return NULL;
			}
		}
                if (bufferGevuld >= 2
			&& result->header[bufferGevuld-1]=='\n'
			&& result->header[bufferGevuld-2]=='\n')
			break;	/* lege regel == einde header */
        }
	result->header[bufferGevuld-2] = 0;	/* nullbyte */
	result->header = realloc(result->header,bufferGevuld-1);
	/* body */
	result->body = malloc(bufferGrootte = 1000);
	if (!result->body) {
		free(result->header); free(result);
		return NULL;
	}
	bufferGevuld = 0;
        while ((letter = fgetc(sock)) != EOF) {
		result->body[bufferGevuld++] = letter;
		if (bufferGevuld == bufferGrootte) {	/* buffer is vol */
			result->body = realloc(result->body,bufferGrootte+=1000);
			if (!result->body) {
				free(result->header); free(result);
				return NULL;
			}
			if (callback) callback(bufferGevuld);
		}
        }
	result->body[bufferGevuld] = 0;	/* nullbyte */
	result->bodyLength = bufferGevuld;
	result->body = realloc(result->body,bufferGevuld+1);
        fclose(sock);
	return result;
}

void freeHttpDocument(struct httpDocument *doc) {
	free(doc->header);
	free(doc->body);
	free(doc);
}
