#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <ctype.h>

#include "gethttp.h"

#define HTTPHEAD "Content-type: text/html\n"
#define TARGET_HOST "zeus.rug.ac.be"
#define TARGET_URL "http://zeus.rug.ac.be"

char *scriptname;

struct tag {
	struct tag *next;
	char *tag;
	char *value;
};

#define printtaglist(t) fprinttaglist(stdout,t)

/* Druk een taglist af, zonder omsluitende '<'...'>' */
void fprinttaglist(FILE *f,struct tag *taglist) {
	while (taglist) {
		if (taglist->value)
			fprintf(f,"%s=\"%s\"",taglist->tag,taglist->value);
		else fprintf(f,"%s",taglist->tag);
		taglist = taglist->next;
		if (taglist) fputc(' ',f);
	}
}

void freetaglist(struct tag *taglist) {
	while (taglist) {
		struct tag *next = taglist->next;
		free(taglist);
		taglist = next;
	}
}

/* *scan wijst naar de tweede letter van een "<blabla ..." string;
 * maak een taglist en geef als resultaat een pointer naar de char
 * vlak na de afsluitende '>' van deze tag */
char *htmlparse(char *scan,struct tag **taglist) {
	struct tag *lasttag = NULL;

	while (1) {
		char *start,*stop;
		struct tag *nieuwetag;
		while (isspace(*scan)) scan++;
		if (*scan == '>') break;
		nieuwetag = malloc(sizeof(struct tag));
		/* tag name */
		start = stop = scan;
		while (*stop && !isspace(*stop) && *stop!='>' && *stop!='=')
			stop++;
		/* tag is opgeslagen in start...stop[-1] */
		nieuwetag->tag = malloc(stop-start+1);
		strncpy(nieuwetag->tag,start,stop-start);
		nieuwetag->tag[stop-start] = 0;
		/* tag value */
		scan = stop;	
		while (isspace(*scan)) scan++;
		if (*scan == '=') {
			/* value */
			scan++;		/* skip '=' */
			while (isspace(*scan)) scan++;
			if (*scan == '"') {
				start = stop = scan+1;
				while (*stop && *stop!='"') stop++;
				scan = stop + 1;
			} else {
				start = stop = scan;
				while (*stop && *stop!='>' && !isspace(*stop))
					stop++;
				scan = stop;
			}
			nieuwetag->value = malloc(stop-start+1);
			strncpy(nieuwetag->value,start,stop-start);
			nieuwetag->value[stop-start] = 0;
		} else nieuwetag->value = NULL;
		/* nieuwetag linken aan de lijst */
		if (lasttag) {
			lasttag->next = nieuwetag;
		} else {
			*taglist = nieuwetag;
		}
		lasttag = nieuwetag;
	}
	if (lasttag) lasttag->next = NULL;
	if (*scan == '>') scan++;
	return scan;
}

/* *scan wijst naar een "<blabla ..." string; verwerk deze tag
 * en geef als resultaat een pointer naar de char vlak na de afsluitende
 * '>' van deze tag */
char *dotag(char *scan) {
	putchar(*scan++);	/* skip '<' */
	while (isspace(*scan)) putchar(*scan++);


	if (!strncasecmp(scan,"a",1)) {
		struct tag *taglist,*snuffel;
		scan = htmlparse(scan,&taglist);
		for (snuffel = taglist; snuffel; snuffel = snuffel->next) {
			if (!strcasecmp(snuffel->tag,"href") && snuffel->value
				&& snuffel->value[0]=='/') {
				printf("%s=\"%s%s\"",
					snuffel->tag,scriptname,snuffel->value);
			} else if (snuffel->value)
				printf("%s=\"%s\"",snuffel->tag,snuffel->value);
			else printf("%s",snuffel->tag);
			if (snuffel->next) putchar(' ');
		}
		freetaglist(taglist);
		putchar('>');
	} else if (!strncasecmp(scan,"img",3)) {
		struct tag *taglist,*snuffel;
		scan = htmlparse(scan,&taglist);
		for (snuffel = taglist; snuffel; snuffel = snuffel->next) {
			if (!strcasecmp(snuffel->tag,"src") && snuffel->value
				&& snuffel->value[0]=='/') {
				printf("%s=\"%s%s\"",
					snuffel->tag,scriptname,snuffel->value);
			} else if (snuffel->value)
				printf("%s=\"%s\"",snuffel->tag,snuffel->value);
			else printf("%s",snuffel->tag);
			if (snuffel->next) putchar(' ');
		}
		freetaglist(taglist);
		putchar('>');
	} else if (!strncasecmp(scan,"body",4)) {
		struct tag *taglist,*snuffel;
		scan = htmlparse(scan,&taglist);
		for (snuffel = taglist; snuffel; snuffel = snuffel->next) {
			if (!strcasecmp(snuffel->tag,"background") && snuffel->value
				&& snuffel->value[0]=='/') {
				printf("%s=\"%s%s\"",
					snuffel->tag,scriptname,snuffel->value);
			} else if (snuffel->value)
				printf("%s=\"%s\"",snuffel->tag,snuffel->value);
			else printf("%s",snuffel->tag);
			if (snuffel->next) putchar(' ');
		}
		freetaglist(taglist);
		putchar('>');
	} else {		/* default */
		while (*scan) {
			putchar(*scan++);
			if (*scan == '>') return scan;
		}
	}
	return scan;
}

int main(int argc,char **argv) {
	char *path = getenv("PATH_INFO");
	struct httpDocument *doc;
	int html = 0;

	if (!path) {
		puts(HTTPHEAD "\n<H1>Error</H1>No PATH_INFO found");
		return 10;
	}

	doc = gethttp(gethostbyname(TARGET_HOST),path,NULL);

	/** content-type bepalen */
	{	char *type = strstr(doc->header,"Content-Type");

		if (!type) puts("Content-Type: text/html\n");
		else {
			if (!strncmp(type+14,"text/html",9)) html = 1;
			while (*type!='\n' && *type!='\r' && *type)
				putchar(*type++);
			puts("\n");
		}
	}

	if (html) {
		/* Alle links naar /... moeten aangepast worden */
		char *scan = doc->body;
		scriptname = getenv("SCRIPT_NAME");
		if (!scriptname) scriptname = "";	/* tsja */
		while (*scan) {
			if (*scan == '<')
				scan = dotag(scan);
			else putchar(*scan++);
		}
	} else {
		fwrite(doc->body,doc->bodyLength,1,stdout);
	}

	return 0;
}
